"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var Ajv = require("ajv");
var ajvKeywords = require("ajv-keywords");
var metaSchema6 = require("ajv/lib/refs/json-schema-draft-06.json");
var every = require("lodash/every");
var findIndex = require("lodash/findIndex");
var isArray = require("lodash/isArray");
var map = require("lodash/map");
var pickBy = require("lodash/pickBy");
var utils = require("../../utils");
var DataTypes_1 = require("../DataTypes");
var ajv = new Ajv();
ajvKeywords(ajv);
ajv.addMetaSchema(metaSchema6);
var FULL_TEXT_SLUG = 'full_text_search';
exports.filter = function (filters, collection) {
    ajv.removeSchema(/^.*$/);
    var validators = isArray(filters)
        ? filters.map(function (s) { return ajv.compile(s); })
        : [ajv.compile(filters)];
    if (isArray(collection)) {
        return collection.filter(function (m) { return every(validators, function (v) { return v(m); }); });
    }
    return pickBy(collection, function (m) { return every(validators, function (v) { return v(m); }); });
};
exports.createFullTextSearchFilter = function (schema, term) {
    var filter = {
        $id: utils.randomString(),
        title: FULL_TEXT_SLUG,
        anyOf: [
            {
                description: "Full text search is " + term,
                anyOf: Object.keys(schema.properties).map(function (key) {
                    return ({
                        properties: (_a = {},
                            _a[key] = {
                                type: 'string',
                                regexp: {
                                    pattern: utils.regexEscape(term),
                                    flags: 'i',
                                },
                            },
                            _a),
                        required: [key],
                    });
                    var _a;
                }),
            },
        ],
    };
    return filter;
};
exports.upsertFullTextSearch = function (schema, filters, term) {
    if (filters === void 0) { filters = []; }
    var searchFilter = exports.createFullTextSearchFilter(schema, term);
    var existingSearchIndex = findIndex(filters, { title: FULL_TEXT_SLUG });
    if (existingSearchIndex > -1) {
        filters.splice(existingSearchIndex, 1, searchFilter);
    }
    else {
        filters.push(searchFilter);
    }
    return filters;
};
exports.removeFullTextSearch = function (filters) {
    return filters.filter(function (f) { return f.title !== FULL_TEXT_SLUG; });
};
exports.createFilter = function (schema, signatures) {
    var anyOf = signatures.map(function (_a) {
        var field = _a.field, operator = _a.operator, value = _a.value;
        var subSchema = schema.properties[field];
        var model = DataTypes_1.getDataModel(subSchema);
        if (!model || !model.operators.hasOwnProperty(operator)) {
            return {};
        }
        return model.createFilter(field, operator, value, subSchema);
    });
    return {
        $id: utils.randomString(),
        anyOf: anyOf,
    };
};
exports.decodeFilter = function (schema, filter) {
    var signatures = filter.anyOf.map(function (f) {
        var schemaField = Object.keys(f.properties).shift();
        var subSchema = schema.properties[schemaField];
        var model = DataTypes_1.getDataModel(subSchema);
        if (!model) {
            return {};
        }
        return model.decodeFilter(f);
    });
    return signatures.filter(function (s) { return s !== null; });
};
exports.getOperators = function (schema, field) {
    var subSchema = schema.properties[field];
    var model = DataTypes_1.getDataModel(subSchema);
    if (!model) {
        return [];
    }
    return map(model.operators, function (value, key) { return ({
        slug: key,
        label: value.getLabel(subSchema),
    }); });
};
