"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var isString = require("lodash/isString");
var noop = require("lodash/noop");
var utils_1 = require("./utils");
var TARGET_OFFSET = 3;
var CLICK_TIMEOUT = 1000;
var arrowStyle = "\n\tposition: absolute;\n\twidth: 0;\n\theight: 0;\n\tborder-color: transparent;\n\tborder-style: solid;\n";
var topArrowStyle = "\n\t" + arrowStyle + "\n\tbottom: 0;\n\tleft: 50%;\n\tmargin-left: -5px;\n\tborder-width: 5px 5px 0;\n\tborder-top-color: #000;\n";
var rightArrowStyle = "\n\t" + arrowStyle + "\n\ttop: 50%;\n\tleft: 0;\n\tmargin-top: -5px;\n\tborder-width: 5px 5px 5px 0;\n\tborder-right-color: #000;\n";
var leftArrowStyle = "\n\t" + arrowStyle + "\n\ttop: 50%;\n\tright: 0;\n\tmargin-top: -5px;\n\tborder-width: 5px 0 5px 5px;\n\tborder-left-color: #000;\n";
var bottomArrowStyle = "\n\t" + arrowStyle + "\n\ttop: 0;\n  left: 50%;\n  margin-left: -5px;\n  border-width: 0 5px 5px;\n  border-bottom-color: #000;\n";
var getArrowStyle = function (placement) {
    switch (placement) {
        case 'right':
            return rightArrowStyle;
        case 'bottom':
            return bottomArrowStyle;
        case 'left':
            return leftArrowStyle;
        default:
            return topArrowStyle;
    }
};
var Tooltips = (function () {
    function Tooltips() {
        this.initialised = false;
    }
    Tooltips.prototype.initialiseElements = function () {
        if (this.initialised || !document || !document.body) {
            return;
        }
        var tooltipElement = document.createElement('div');
        tooltipElement.setAttribute('style', "\n\t\t\tdisplay: block;\n\t\t\tfont-size: 12px;\n\t\t\tfont-style: normal;\n\t\t\tfont-weight: 400;\n\t\t\tletter-spacing: normal;\n\t\t\tline-break: auto;\n\t\t\tline-height: 1.42857143;\n\t\t\tpadding: 5px;\n\t\t\tposition: absolute;\n\t\t\ttext-align: left;\n\t\t\ttext-align: start;\n\t\t\ttext-decoration: none;\n\t\t\ttext-shadow: none;\n\t\t\ttext-transform: none;\n\t\t\tvisibility: hidden;\n\t\t\twhite-space: normal;\n\t\t\tword-break: normal;\n\t\t\tword-spacing: normal;\n\t\t\tword-wrap: normal;\n\t\t\tz-index: 1070;\n\t\t");
        var tooltipElementInner = document.createElement('div');
        tooltipElementInner.setAttribute('style', "\n\t\t\tbackground: black;\n\t\t\tborder-radius: 4px;\n\t\t\tcolor: white;\n\t\t\tpadding: 3px 8px;\n\t\t\ttextAlign: center;\n\t\t");
        var tooltipArrow = document.createElement('div');
        tooltipArrow.setAttribute('style', "\n\t\t\t" + arrowStyle + "\n\t\t\tbottom: 0;\n\t\t\tleft: 50%;\n\t\t\tmargin-left: -5px;\n\t\t\tborder-width: 5px 5px 0;\n\t\t\tborder-top-color: #000;\n\t\t");
        tooltipElement.appendChild(tooltipElementInner);
        tooltipElement.appendChild(tooltipArrow);
        document.body.appendChild(tooltipElement);
        this.tooltipElement = tooltipElement;
        this.tooltipElementInner = tooltipElementInner;
        this.tooltipElementArrow = tooltipArrow;
        this.initialised = true;
    };
    Tooltips.prototype.bindProps = function (props) {
        var _this = this;
        if (props.tooltip) {
            var trigger = 'hover';
            var tooltipText_1;
            var placement_1;
            if (isString(props.tooltip)) {
                tooltipText_1 = props.tooltip;
            }
            else {
                tooltipText_1 = props.tooltip.text;
                placement_1 = props.tooltip.placement;
                if (props.tooltip.trigger) {
                    trigger = props.tooltip.trigger;
                }
            }
            if (tooltipText_1) {
                var showFn_1 = function (e) { return _this.show(e, tooltipText_1, placement_1); };
                if (trigger === 'click') {
                    var oldFn_1 = props.onClick || noop;
                    var hideFn_1 = function () {
                        clearTimeout(_this.hideTimeout);
                        _this.hideTimeout = window.setTimeout(function () { return _this.hide(); }, CLICK_TIMEOUT);
                    };
                    props.onClick = function (e) { return showFn_1(e) || hideFn_1() || oldFn_1(e); };
                }
                else {
                    var oldMEFn_1 = props.onMouseEnter || noop;
                    props.onMouseEnter = function (e) { return showFn_1(e) || oldMEFn_1(e); };
                    var oldMLFn_1 = props.onMouseLeave || noop;
                    props.onMouseLeave = function (e) { return _this.hide() || oldMLFn_1(e); };
                    props.onMouseLeave = function () { return _this.hide(); };
                }
            }
        }
        return props;
    };
    Tooltips.prototype.show = function (e, tooltipText, placement) {
        this.initialiseElements();
        this.tooltipElementInner.innerText = tooltipText;
        this.tooltipElementArrow.setAttribute('style', getArrowStyle(placement));
        var boundingRect = e.target.getBoundingClientRect();
        if (!placement || placement === 'top') {
            this.tooltipElement.style.top = utils_1.px(boundingRect.top - this.tooltipElement.clientHeight - TARGET_OFFSET);
            this.tooltipElement.style.left = utils_1.px(boundingRect.left +
                boundingRect.width / 2 -
                this.tooltipElement.clientWidth / 2);
        }
        if (placement === 'right') {
            this.tooltipElement.style.top = utils_1.px(boundingRect.top +
                boundingRect.height / 2 -
                this.tooltipElement.clientHeight / 2);
            this.tooltipElement.style.left = utils_1.px(boundingRect.left + boundingRect.width + TARGET_OFFSET);
        }
        if (placement === 'bottom') {
            this.tooltipElement.style.top = utils_1.px(boundingRect.top + boundingRect.height + TARGET_OFFSET);
            this.tooltipElement.style.left = utils_1.px(boundingRect.left +
                boundingRect.width / 2 -
                this.tooltipElement.clientWidth / 2);
        }
        if (placement === 'left') {
            this.tooltipElement.style.top = utils_1.px(boundingRect.top +
                boundingRect.height / 2 -
                this.tooltipElement.clientHeight / 2);
            this.tooltipElement.style.left = utils_1.px(boundingRect.left - this.tooltipElement.clientWidth - TARGET_OFFSET);
        }
        this.tooltipElement.style.visibility = 'visible';
    };
    Tooltips.prototype.hide = function () {
        this.initialiseElements();
        this.tooltipElement.style.visibility = 'hidden';
    };
    return Tooltips;
}());
exports.Tooltips = Tooltips;
